<?php
/**
 * Joomla! 1.5 component Simple Lists
 *
 * @author Jisse Reitsma (info@jira.nl)
 * @package SimpleLists
 * @copyright Copyright 2007
 * @license GNU Public License
 * @link http://www.jira.nl
 * @version 1.5.0.1
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport( 'joomla.application.component.controller' );
require_once( JPATH_COMPONENT.DS.'helpers'.DS.'helper.php' );

/**
 * Simplelists Controller
 *
 * @package Joomla
 * @subpackage Simplelists
 * @since 1.5
 */
class SimplelistsController extends JController
{

    /**
     * Constructor
     * @access private
     * @subpackage SimpleLists
     */
    function __construct()
    {
        // Make sure we have set the proper view
        $this->_setViewType();

        parent::__construct();
    }


    /**
     * Display the current page
     * @access public
     * @subpackage SimpleLists
     */
    function display( )
    {
        parent::display();
    }


    /**
     * Handle the task 'add'
     * @access private
     * @subpackage SimpleLists
     */
    function add()
    {
        $this->_setForm() ;
        JRequest::setVar( 'edit', false );
        parent::display();
    }


    /**
     * Handle the task 'edit'
     * @access private
     * @subpackage SimpleLists
     */
    function edit()
    {
        $this->_setForm() ;
        JRequest::setVar( 'edit', true );
        $model = $this->_loadModel();
        $model->checkout();
        parent::display();
    }


    /**
     * Handle the task 'store'
     * @access private
     * @subpackage SimpleLists
     */
    function store()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );
        $post    = JRequest::get('post');
        $cid    = JRequest::getVar( 'cid', array(0), 'post', 'array' );
        $post['id'] = (int)$cid[0];
        $this->id = $post['id'] ;
        $post['text'] = JRequest::getVar('text', '', 'post', 'string', JREQUEST_ALLOWRAW);
        $post['description'] = JRequest::getVar('description', '', 'post', 'string', JREQUEST_ALLOWRAW);

        $model = $this->_loadModel();

        if ($model->store($post)) {
            $this->msg = JText::_( $this->item_type . ' Saved' );
        } else {
            $this->msg = JText::_( 'Error Saving '.$this->item_type );
        }

        // Check the table in so it can be edited.... we are done with it anyway
        $model->checkin();
    }


    /**
     * Handle the task 'save'
     * @access private
     * @subpackage SimpleLists
     */
    function save()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );
        $this->store() ;

        $view = (JRequest::getVar('view') == 'category') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link, $this->msg);
    }


    /**
     * Handle the task 'apply'
     * @access private
     * @subpackage SimpleLists
     */
    function apply()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );
        $this->store() ;

        $view = (JRequest::getVar('view') == 'category') ? 'category' : 'item' ;
        $link = 'index.php?option=com_simplelists&view='.$view.'&task=edit&cid[]=' . $this->id ;
        $this->setRedirect($link, $this->msg);
    }


    /**
     * Handle the task 'remove'
     * @access private
     * @subpackage SimpleLists
     */
    function remove()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );
        $cid = JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($cid);

        if (count( $cid ) < 1) {
            JError::raiseError(500, JText::_( 'Select an item to delete' ) );
        }

        $model = $this->_loadModel();
        if(!$model->delete($cid)) {
            $this->msg = $model->getError() ;
        } else {
            $this->msg = JText::sprintf( $this->item_type . ' Removed', count($cid) );
        }

        $view = (JRequest::getVar('view') == 'categories') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link, $this->msg );
    }


    /**
     * Handle the task 'publish'
     * @access private
     * @subpackage SimpleLists
     */
    function publish()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );
        $cid = JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($cid);

        if (count( $cid ) < 1) {
            JError::raiseError(500, JText::_( 'Select an item to publish' ) );
        }

        $model = $this->_loadModel();
        if(!$model->publish($cid, 1)) {
            echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
        } else {
            $this->msg = JText::sprintf( $this->item_type . ' Published', count($cid) );
        }

        $view = (JRequest::getVar('view') == 'categories') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link, $this->msg );
    }


    /**
     * Handle the task 'unpublish'
     * @access private
     * @subpackage SimpleLists
     */
    function unpublish()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );
        $cid = JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($cid);

        if (count( $cid ) < 1) {
            JError::raiseError(500, JText::_( 'Select an item to unpublish' ) );
        }

        $model = $this->_loadModel();
        if(!$model->publish($cid, 0)) {
            echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
        } else {
            $this->msg = JText::sprintf( $this->item_type . ' Unpublished', count($cid) );
        }

        $view = (JRequest::getVar('view') == 'categories') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link, $this->msg );
    }


    /**
     * Handle the task 'cancel'
     * @access private
     * @subpackage SimpleLists
     */
    function cancel()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );

        // Checkin the model
        $model = $this->_loadModel();
        $model->checkin();

        $view = (JRequest::getVar('view') == 'category') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link );
    }


    /**
     * Handle the task 'orderup'
     * @access private
     * @subpackage SimpleLists
     */
    function orderup()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );

        $model = $this->_loadModel();
        $model->move(-1);

        $view = (JRequest::getVar('view') == 'categories') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link );
    }


    /**
     * Handle the task 'orderdown'
     * @access private
     * @subpackage SimpleLists
     */
    function orderdown()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );

        $model = $this->_loadModel();
        $model->move(1);

        $view = (JRequest::getVar('view') == 'categories') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link );
    }


    /**
     * Handle the task 'saveorder'
     * @access private
     * @subpackage SimpleLists
     */
    function saveorder()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );
        $cid = JRequest::getVar( 'cid', array(), 'post', 'array' );
        $order = JRequest::getVar( 'order', array(), 'post', 'array' );
        JArrayHelper::toInteger($cid);
        JArrayHelper::toInteger($order);

        $model = $this->_loadModel();
        $model->saveorder($cid, $order);

        $view = (JRequest::getVar('view') == 'categories') ? 'categories' : 'items' ;
        $link = 'index.php?option=com_simplelists&view='.$view ;
        $this->setRedirect( $link );
    }


    /**
     * Helper function to set a type-string for later use
     * @access private
     * @subpackage SimpleLists
     */
    function _setViewType()
    {
        switch(JRequest::getCmd( 'view'))
        {
            case 'category':
                $this->item_type = 'Category' ;
                break ;
            case 'categories':
                $this->item_type = 'Categories' ;
                break ;
            case 'item':
                $this->item_type = 'Item' ;
                break ;
            default:
                $this->item_type = 'Items' ;
                break ;
        }
    }


    /**
     * Helper function to set the form page
     * @access private
     * @subpackage SimpleLists
     */
    function _setForm() {
        switch(JRequest::getCmd('view')) {
            case 'category':
            case 'categories':
                JRequest::setVar('view', 'category');
                break ;
            default:
                JRequest::setVar('view', 'item');
                break ;
        }

        // Hide the menu while editing or adding an item
        JRequest::setVar( 'hidemainmenu', 1 );

        // Set the layout template to 'form'
        JRequest::setVar( 'layout', 'form' );
    }


    /**
     * Helper function to load the current model
     * @access private
     * @subpackage SimpleLists
     */
    function _loadModel()
    {
        switch(JRequest::getVar('view')) {
            case 'category':
            case 'categories':
                $model = 'category';
                break ;
            default:
                $model = 'item' ;
                break ;
        }
        return $this->getModel( $model );
    }
}
