<?php
/**
 * Joomla! 1.5 component Simple Lists
 *
 * @author Jisse Reitsma (info@jira.nl)
 * @package SimpleLists
 * @copyright Copyright 2007
 * @license GNU Public License
 * @link http://www.jira.nl
 * @version 1.5.0.1
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Simplelists Helper
 * 
 * @package Joomla
 * @subpackage Simplelists
 * @since 1.5
 */
class SimplelistsHelper
{
    function getLinkType( $int = 0 ) {
        switch( $int ) {
            case 1: 
                $text = JText::_( 'Custom link' );
                break ;
            case 2: 
                $text = JText::_( 'Internal menu-link' );
                break ;
            case 3: 
                $text = JText::_( 'Internal article' );
                break ;
            case 4: 
                $text = JText::_( 'Internal image' );
                break ;
            case 5: 
                $text = JText::_( 'Internal file' );
                break ;
            default:
                $text = JText::_( 'None' );
                break ;
        }
        return $text ;
    }

    function getLinkName( $link_type, $link ) {
        switch( $link_type ) {
            case 0:
                return '';
            case 2:
                return SimplelistsHelper::getMenuName( $link );
            case 3:
                return SimplelistsHelper::getArticleName( $link );
            case 1:
            case 4:
            case 5:
                return $link ;
        }
    }

    function getUrl( $link_type, $link ) {
        switch( $link_type ) {
            case 0:
                return '';
            case 2:
                $link = SimplelistsHelper::getMenuLink( $link );
                return $link;
            case 3:
                require_once JPATH_SITE.DS.'components'.DS.'com_content'.DS.'helpers'.DS.'route.php' ;
                $link = ContentHelperRoute::getArticleRoute((int)$link);
                return $link;
            case 1:
            case 4:
            case 5:
                return $link;
        }
    }

    function getCategories( $id = null, $parent_id = null ) {
        if( is_numeric( $id )) {
            $query = 'SELECT c.* FROM `#__simplelists_categories` AS s LEFT JOIN `#__categories`  AS c ON c.`id` = s.`category_id` WHERE s.`id`='.(int)$id ;
        } elseif( (int)$parent_id > 0 ) {
            $query = 'SELECT c.* FROM `#__categories` AS c WHERE c.`section` = "com_simplelists" AND c.`parent_id` = '.(int)$parent_id ;
        } else {
            $query = 'SELECT c.* FROM `#__categories` AS c WHERE c.`section` = "com_simplelists"' ;
        }
        $db =& JFactory::getDBO();
        $db->setQuery( $query ) ;
        $rows = $db->loadObjectList() ;
        return $rows ;
    }

    function getCategoryOptions( $id = 0, $parent_id = 0 ) {
        $categories = SimplelistsHelper::getCategories( null, $parent_id ) ;
        $current = SimplelistsHelper::getCategories( $id ) ;
        if( $id == 0 && count($categories) == 1 ) {
            $current = $categories ;
        }
        $size = (count($categories) < 10) ? count($categories) : 10 ;
		return JHTML::_('select.genericlist', $categories, 'categories[]', 'size="'.$size.'" multiple', 'id', 'title', $current );
    }

    function getNumItems( $category_id ) {
        $query = 'SELECT id FROM `#__simplelists_categories` WHERE `category_id`='.(int)$category_id ;
        $db =& JFactory::getDBO();
        $db->setQuery( $query ) ;
        $rows = $db->loadAssocList() ;
        return count($rows);
    }

    function createDir( $folder = '' ) {

        $application = JFactory::getApplication() ;

        if( $folder == '' ) {
            $folder = JPATH_SITE.DS.'images'.DS.'simplelists' ;
        }
        if( JFolder::exists( $folder )) {
            return true ;
        } else {
            if( JFolder::create( $folder )) {
                $application->enqueueMessage( JText::_( 'Created image directory' ) . ": $folder", 'notice' ) ;
                return true ;
            } else {
                $application->enqueueMessage( JText::_( 'Failed to create directory' ) . ": $folder", 'error' ) ;
                return false ;
            }
        }
    }

    function checkCategories() {

        $application = JFactory::getApplication() ;

        $db =& JFactory::getDBO();
        $db->setQuery( "SELECT * FROM #__categories WHERE `section`='com_simplelists'" ) ;
        $rows = $db->loadAssocList() ;
        if(empty($rows)) {
            $application->enqueueMessage( JText::_( 'You have no categories configured yet' ), 'notice' ) ;
            return false ;
        }
        return true ;
    }

    function getArticleName( $article_id ) {
        $query = "SELECT `title` FROM #__content WHERE `id`=".(int)$article_id ;
        $db =& JFactory::getDBO();
        $db->setQuery( $query );
        $row = $db->loadObject() ;
        if(is_object($row)) {
            return $row->title ;
        } else {
            return '' ;
        }
    }

    function getMenuName( $menu_id ) {
        $query = "SELECT `name` FROM #__menu WHERE `id`=".(int)$menu_id ;
        $db =& JFactory::getDBO();
        $db->setQuery( $query );
        $row = $db->loadObject() ;
        if(is_object($row)) {
            return $row->name;
        } else {
            return '' ;
        }
    }

    function getMenuLink( $menu_id ) {
        $query = "SELECT `link` FROM #__menu WHERE `id`=".(int)$menu_id ;
        $db =& JFactory::getDBO();
        $db->setQuery( $query );
        $row = $db->loadObject() ;
        if(is_object($row)) {
            return JRoute::_( $row->link . '&Itemid='.(int)$menu_id );
        } else {
            return '' ;
        }
    }
}
