<?php
/**
 * Joomla! 1.5 component Simple Lists
 *
 * @author Jisse Reitsma (info@jira.nl)
 * @package SimpleLists
 * @copyright Copyright 2007
 * @license GNU Public License
 * @link http://www.jira.nl
 * @version 1.5.0.1
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport('joomla.application.component.model');
jimport('joomla.utilities.date');

class SimplelistsModelItem extends JModel
{
	/**
	 * Item id
	 *
	 * @var int
	 */
	var $_id = null;

	/**
	 * Item data
	 *
	 * @var array
	 */
	var $_data = null;

	/**
	 * Constructor
	 *
	 * @since 1.5
	 */
	function __construct()
	{
		parent::__construct();

		$array = JRequest::getVar('cid', array(0), '', 'array');
		$edit	= JRequest::getVar('edit',true);
		if($edit) {
			$this->setId((int)$array[0]);
        }
	}

	/**
	 * Method to set the item identifier
	 *
	 * @access	public
	 * @param	int Item identifier
	 */
	function setId($id)
	{
		// Set item id and wipe data
		$this->_id		= $id;
		$this->_data	= null;
	}

	/**
	 * Method to get an item
	 *
	 * @since 1.5
	 */
	function &getData()
	{
		// Load the item data
		if (!$this->_loadData())
		{
		    $this->_initData();
		}
		return $this->_data;
	}

	/**
	 * Tests if item is checked out
	 *
	 * @access	public
	 * @param	int	A user id
	 * @return	boolean	True if checked out
	 * @since	1.5
	 */
	function isCheckedOut( $uid=0 )
	{
		if ($this->_loadData())
		{
			if ($uid) {
				return ($this->_data->checked_out && $this->_data->checked_out != $uid);
			} else {
				return $this->_data->checked_out;
			}
		}
	}

	/**
	 * Method to checkin/unlock the item
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function checkin()
	{
		if ($this->_id)
		{
			$item = & $this->getTable();
			if(! $item->checkin($this->_id)) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}
		return false;
	}

	/**
	 * Method to checkout/lock the item
	 *
	 * @access	public
	 * @param	int	$uid	User ID of the user checking the article out
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function checkout($uid = null)
	{
		if ($this->_id)
		{
			// Make sure we have a user id to checkout the article with
			if (is_null($uid)) {
				$user	=& JFactory::getUser();
				$uid	= $user->get('id');
			}

			// Checkout the table
			$item = & $this->getTable();
			if(!$item->checkout($uid, $this->_id)) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}

			return true;
		}
		return false;
	}

	/**
	 * Method to store the item
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function store($data)
	{
		$row =& $this->getTable();

        // Insert $categories manually
        if(!empty( $data['categories'] )) {
            $categories = $data['categories'] ;
            unset($data['categories']) ;
        }

        // Insert $link manually
        if($data['link_type'] == 1 && !empty($data['link_external'])) {
            $data['link'] = $data['link_external'] ;
        } elseif($data['link_type'] == 2 && !empty($data['link_menu'])) {
            $data['link'] = $data['link_menu'] ;
        } elseif($data['link_type'] == 3 && !empty($data['link_article'])) {
            $data['link'] = $data['link_article'] ;
        } elseif($data['link_type'] == 4 && !empty($data['link_image'])) {
            $data['link'] = $data['link_image'] ;
        } elseif($data['link_type'] == 5 && !empty($data['link_file'])) {
            $data['link'] = $data['link_file'] ;
        } else {
            $data['link_type'] = 0 ;
            $data['link'] = '' ;
        }

        // Set the user metadata
        $now = new JDate('now');
		$user =& JFactory::getUser();
		$uid = $user->get('id');
        $data['modified'] = $now->toMySQL();
        $data['modified_by'] = $uid ;
        if( $data['id'] == 0 ) {
            $data['created'] = $now->toMySQL();
            $data['created_by'] = $uid ;
        }

		// Bind the form fields to the item table
		if (!$row->bind($data)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Make sure the item table is valid
		if (!$row->check()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Store the item table to the database
		if (!$row->store()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

        // Store the categories
        $query = 'DELETE FROM `#__simplelists_categories` WHERE id='.(int)$row->id ;
		$this->_db->setQuery( $query );
		$this->_db->query() ;
        foreach( $categories as $c ) {
            $query = 'INSERT INTO `#__simplelists_categories` SET id='.(int)$row->id.', category_id='.(int)$c ;
		    $this->_db->setQuery( $query );
    		$this->_db->query() ;
        }

		return true;
	}

	/**
	 * Method to remove an item
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function delete($cid = array())
	{
		$result = false;

		if (count( $cid ))
		{
			JArrayHelper::toInteger($cid);
			$cids = implode( ',', $cid );
			$query = 'DELETE FROM #__simplelists WHERE id IN ( '.$cids.' )';
			$this->_db->setQuery( $query );
			if(!$this->_db->query()) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}

		return true;
	}

	/**
	 * Method to (un)publish an item
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function publish($cid = array(), $publish = 1)
	{
		$user 	=& JFactory::getUser();

		if (count( $cid ))
		{
			JArrayHelper::toInteger($cid);
			$cids = implode( ',', $cid );

			$query = 'UPDATE #__simplelists'
				. ' SET published = '.(int) $publish
				. ' WHERE id IN ( '.$cids.' )'
				. ' AND ( checked_out = 0 OR ( checked_out = '.(int) $user->get('id').' ) )'
			;
			$this->_db->setQuery( $query );
			if (!$this->_db->query()) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}

		return true;
	}

	/**
	 * Method to move an item
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function move($direction)
	{
		$row =& $this->getTable();
		if (!$row->load($this->_id)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		if (!$row->move( $direction, ' published >= 0 ' )) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		return true;
	}

	/**
	 * Method to move an item
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function saveorder($cid = array(), $order)
	{
		$row =& $this->getTable();
		$groupings = array();

		// update ordering values
		for( $i=0; $i < count($cid); $i++ )
		{
			$row->load( (int) $cid[$i] );

			if ($row->ordering != $order[$i])
			{
				$row->ordering = $order[$i];
				if (!$row->store()) {
					$this->setError($this->_db->getErrorMsg());
					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Method to load content item data
	 *
	 * @access	private
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function _loadData()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{
			$query = 'SELECT w.*' .
					' FROM #__simplelists AS w' .
					' WHERE w.id = '.(int) $this->_id ;
			$this->_db->setQuery($query);
			$this->_data = $this->_db->loadObject();

            if( $this->_data ) {
                
                // Fetch the extra link data
                if( $this->_data->link_type == 1 ) {
                    $this->_data->custom_link = $this->_data->link ;
                } else {
                    $this->_data->custom_link = '' ;
                }

                if( $this->_data->link_type == 2 ) {
                    $this->_data->menu_name = SimplelistsHelper::getMenuName( $this->_data->link ) ;
                } else {
                    $this->_data->menu_name = '' ;
                }

                if( $this->_data->link_type == 3 ) {
                    $this->_data->article_name = SimplelistsHelper::getArticleName( $this->_data->link ) ;
                } else {
                    $this->_data->article_name = '' ;
                }

                if( $this->_data->link_type == 4 ) {
                    $this->_data->image_name = $this->_data->link ;
                } else {
                    $this->_data->image_name = '' ;
                }

                if( $this->_data->link_type == 5 ) {
                    $this->_data->file_name = $this->_data->link ;
                } else {
                    $this->_data->file_name = '' ;
                }

            }

			return (boolean) $this->_data;
		}
		return true;
	}

	/**
	 * Method to initialise the item data
	 *
	 * @access	private
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function _initData()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{
			$item = new stdClass();
			$item->id					    = 0;
			$item->custom_link              = null;
			$item->article_name		        = null;
			$item->image_name		        = null;
			$item->file_name		        = null;
			$item->created                  = null;
			$item->created_by               = 0;
			$item->created_by_alias         = null;
			$item->modified                 = null;
			$item->modified_by              = 0;
			$item->modified_by_alias        = null;
			$item->title				    = null;
			$item->link_type                = 0; 
			$item->link				        = null;
			$item->text                     = null;
			$item->published			    = 0;
			$item->checked_out		        = 0;
			$item->checked_out_time	        = 0;
			$item->ordering			        = 0;
			$item->hits                     = 0;
			$item->params				    = null;
			$item->picture                  = null;
			$this->_data					= $item;
			return (boolean) $this->_data;
		}
		return true;
	}
}
