<?php
/**
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport('joomla.application.component.model');

/**
 * Simplelists Component Categories Model
 *
 * @author      Jisse Reitsma
 * @package		Jira
 * @subpackage	Simplelists
 * @since 1.5
 */
class SimplelistsModelCategories extends JModel
{
	/**
	 * Simplelist categories
	 *
	 * @var array
	 */
	var $_categories = null;

	/**
	 * Parent category ID
	 *
	 * @var int
	 */
	var $_parent_id = null;

	/**
	 * Parent category
	 *
	 * @var object
	 */
	var $_parent = null;

    /**
     * Template total
     *
     * @var integer
     */
    var $_total = null;

    /**
     * Pagination object
     *
     * @var object
     */
    var $_pagination = null;

	/**
	 * Constructor
	 *
	 * @since 1.5
	 */
	function __construct()
	{
		parent::__construct();
        global $option;

        $mainframe =& JFactory::getApplication() ;
        $config = JFactory::getConfig() ;

        // Get the pagination request variables
        $limit = $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $config->getValue('config.list_limit'), 'int' );
        $limitstart = JRequest::getInt( 'limitstart', 0 );

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);
	}

	/**
	 * Method to get a list of categories
	 *
	 * @since 1.5
	 */
	function &getCategories()
	{
        $this->loadCategories();
        if( $this->_pagination ) {
            $categories = array_slice( $this->_categories, $this->getState('limitstart'), $this->getState('limit'));
            return $categories ;
        } else {
            return $this->_categories ;
        }
	}

    /**
	 * Method to load category data
	 *
	 * @access	private
	 * @return	boolean	True on success
	 */
	function loadCategories()
	{
		if (empty($this->_categories))
		{
			// current category info
			$query = 'SELECT c.*, ' .
				' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug '.
				' FROM #__categories AS c' .
				' WHERE c.section = "com_simplelists" ' . 
                ' AND c.published = 1'
            ;

            if($this->_parent_id > 0) {
                $query .= ' AND c.parent_id = '.(int)$this->_parent_id ;
            }

			$this->_db->setQuery($query);
			$this->_categories = $this->_db->loadObjectList();
		}
		return true;
	}

    /**
	 * Method to get the fully loaded parent
	 *
	 * @access	private
	 * @return	boolean	True on success
	 */
	function getParent( $parent_id = null )
	{
		if (empty($this->_parent))
		{
            if( $parent_id != null ) {
                $this->_parent_id = (int)$parent_id ;
            }

			// current category info
			$query = 'SELECT c.*, ' .
				' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug '.
				' FROM #__categories AS c' .
				' WHERE c.id = '.(int)$this->_parent_id;
            ;

			$this->_db->setQuery($query);
			$this->_parent = $this->_db->loadObject();

            if( $this->_parent == null ) {
                return $this->_initParent();
            }

		}
		return $this->_parent;
	}

    /**
     * Method to get the total number of simplelist items
     *
     * @access public
     * @return integer
     */
    function getTotal()
    {
        return $this->_total;
    }

    /**
     * Method to get a pagination object for the simplelists
     *
     * @access public
     * @return integer
     */
    function getPagination()
    {
        // Lets load the content if it doesn't already exist
        if (empty($this->_pagination))
        {
            jimport('joomla.html.pagination');
            $this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
        }
        return $this->_pagination;
    }

    /**
     * Method to override the default limit configuration
     *
     * @access public
     * @param integer
     * @return boolean
     */
    function setLimit( $limit ) {
        if( $limit > 0 ) {
            $this->setState('limit', $limit);
            return true ;
        }
        return false ;
    }

    /**
     * Method to set the parent category
     *
     * @access public
     * @param integer
     * @return boolean 
     */
    function setParent( $parent_id )
    {
        if( $parent_id > 0 ) {
            $this->_parent_id = $parent_id ;
            return true ;
        }
        return false ;
    }

    function _initParent() {
        if (empty($this->_parent))
        {
            $category = new stdClass();
            $category->id = 0;
            $category->parent_id = 0;
            $category->title = null;
            $category->name = null;
            $category->alias = null;
            $category->image = null;
            $category->section = null;
            $category->image_position = null;
            $category->description = null;
            $category->published = 0;
            $category->checked_out = 0;
            $category->checked_out_time = 0;
            $category->editor = null;
            $category->ordering = 0;
            $category->access = 0;
            $category->count = 0;
            $category->params = null;
            $this->_parent = $category;
        }
        return $this->_parent;
    }
}
