<?php
/**
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport('joomla.application.component.model');

/**
 * Simplelists Component Simplelist Model
 *
 * @author      Jisse Reitsma
 * @package		Jira
 * @subpackage	Simplelists
 * @since 1.5
 */
class SimplelistsModelSimplelist extends JModel
{
	/**
	 * Simplelist id
	 *
	 * @var int
	 */
	var $_id = null;

	/**
	 * Simplelist category
	 *
	 * @var int
	 */
	var $_category = null;

	/**
	 * Simplelist data
	 *
	 * @var array
	 */
	var $_data = null;

    /**
     * Template total
     *
     * @var integer
     */
    var $_total = null;

    /**
     * Pagination object
     *
     * @var object
     */
    var $_pagination = null;

	/**
	 * Constructor
	 *
	 * @since 1.5
	 */
	function __construct()
	{
		parent::__construct();
        global $option;

        $mainframe =& JFactory::getApplication() ;
         $config = JFactory::getConfig() ;

		$this->setId( JRequest::getInt('category_id', '0' ));

        // Get the pagination request variables
        $limit = $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $config->getValue('config.list_limit'), 'int' );
        $limitstart = JRequest::getInt( 'limitstart', 0 );

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);
	}

	/**
	 * Method to set the simplelist identifier
	 *
	 * @access	public
	 * @param	int Simplelist identifier
	 */
	function setId($id)
	{
		$this->_id		= $id;
	}

	/**
	 * Method to get a simplelist
	 *
	 * @since 1.5
	 */
	function &getData()
	{
		$this->loadData() ;
        if( $this->_pagination ) {
            $data = array_slice( $this->_data, $this->getState('limitstart'), $this->getState('limit'));
            return $data ;
        } else {
            return $this->_data ;
        }
	}

	/**
	 * Method to load content simplelist data
	 *
	 * @access	private
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function loadData()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{

            $where = $this->getWhere() ;
            $orderby = $this->getOrderby() ;
			$query = 'SELECT w.*, c.title AS category'.
					' FROM #__simplelists_categories AS sc' .
					' LEFT JOIN #__simplelists AS w ON w.id = sc.id' .
					' LEFT JOIN #__categories AS c ON c.id = sc.category_id' .
					' WHERE sc.category_id = '.(int) $this->_id .
                    ' AND c.published = 1' .
                    ' AND w.published = 1' .
                    $where .
                    $orderby ;
			$this->_db->setQuery($query);
			$this->_data = $this->_db->loadObjectList();
            $this->_total = count( $this->_data ) ;
			return $this->_data;
		}
		return array() ;
	}

	/**
	 * Method to get a simplelist
	 *
	 * @since 1.5
	 */
	function &getCategory()
	{
		if( ! $this->_category )
		{
            $this->_loadCategory();
        }
		return $this->_category;
	}

    /**
	 * Method to load category data if it doesn't exist.
	 *
	 * @access	private
	 * @return	boolean	True on success
	 */
	function _loadCategory()
	{
		if (empty($this->_category))
		{
			// current category info
			$query = 'SELECT c.*, ' .
				' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug '.
				' FROM #__categories AS c' .
				' WHERE c.id = '. (int) $this->_id .
				' AND c.section = "com_simplelists"' .
                ' AND c.published = 1'
            ;
			$this->_db->setQuery($query, 0, 1);
			$this->_category = $this->_db->loadObject();

            if( $this->_category == null ) {
                return $this->_initCategory();
            }

            // fetch the related categories (parent and children)
			$query = 'SELECT c.*, ' .
				' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as slug '.
				' FROM #__categories AS c' .
				' WHERE (c.id = '. (int) $this->_category->parent_id .
                ' OR c.parent_id = '. (int) $this->_category->id . ')' .
				' AND c.section = "com_simplelists"' .
                ' AND c.published = 1'
            ;
			$this->_db->setQuery($query);
			$related = $this->_db->loadObjectList();
            foreach( $related as $id => $cat ) {
                if( $cat->id == $this->_category->parent_id ) {
                    $this->_category->parent = $cat ;
                    unset( $related[$id] );
                    break ;
                }
            }
            $this->_category->childs = $related ;
		}
		return true;
	}

    /**
     * Method to get the total number of simplelist items
     *
     * @access public
     * @return integer
     */
    function getTotal()
    {
        return $this->_total;
    }

    /**
     * Method to get a pagination object for the simplelists
     *
     * @access public
     * @return integer
     */
    function getPagination()
    {
        // Lets load the content if it doesn't already exist
        if (empty($this->_pagination))
        {
            jimport('joomla.html.pagination');
            $this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
        }
        return $this->_pagination;
    }

    function getWhere() {
        $where = '' ;
        if( preg_match( '/^([a-z]{1})$/', JRequest::getCmd('char'))) {
            $where .= ' AND w.title LIKE \''. JRequest::getCmd('char') . '%\'' ;
        }
        return $where ;
    }

    function getOrderby() {
        $params = &JComponentHelper::getParams( 'com_simplelists' );
        $ordering = $params->get('orderby');
        switch( $ordering ) {
            case 'alpha': 
                $o = 'w.title ASC' ;
                break ;
            case 'ralpha': 
                $o = 'w.title DESC' ;
                break ;
            case 'date': 
                $o = 'w.created DESC, w.modified DESC' ;
                break ;
            case 'rdate': 
                $o = 'w.created ASC, w.modified ASC' ;
                break ;
            default:
                $o = 'w.ordering' ;
                break ;
        }
        return ' ORDER BY '.$o ;
    }

    /**
     * Method to override the default limit configuration
     *
     * @access public
     * @param integer
     * @return boolean
     */
    function setLimit( $limit ) {
        if( $limit > 0 ) {
            $this->setState('limit', $limit);
            return true ;
        }
        return false ;
    }

    function _initCategory() {
        if (empty($this->_category))
        {
            $category = new stdClass();
            $category->id = 0;
            $category->parent_id = 0;
            $category->title = null;
            $category->name = null;
            $category->alias = null;
            $category->image = null;
            $category->section = null;
            $category->image_position = null;
            $category->description = null;
            $category->published = 0;
            $category->checked_out = 0;
            $category->checked_out_time = 0;
            $category->editor = null;
            $category->ordering = 0;
            $category->access = 0;
            $category->count = 0;
            $category->params = null;
            $this->_category = $category;
        }
        return $this->_category;
    }
}
