<?php
/**
 * @package Joomla
 * @copyright 2007 Vitaliy Marenkov
 * @author Vitaliy Marenkov <admin@mavik.com.ua>
 * Плагин заменяет изображения иконками со ссылкой на полную версию.
 */

// Check to ensure this file is included in Joomla!
defined( '_JEXEC' ) or die();

jimport( 'joomla.event.plugin' );

/**
 * Плагин заменяет изображения иконками со ссылкой на полную версию.
 */
class plgContentMavikThumbnails extends JPlugin
{

	/**
	 * XML-элемент тег img
	 * @var JSimpleXMLElement
	 */
	var $_img;
	
	/**
	 * Массив атрибутов тега img
	 * @var array
	 */
	var $_imgAtributes;
	
	/**
	 * Имя оригинального изображения
	 * @var string
	 */
	var $_origImgName;
	
	/**
	 * Размеры оригинального изображения
	 * @var array
	 */
	var $_origImgSize;
	
	/**
	 * Каталог с иконками
	 * @var string
	 */
	var $_thumbPath;
	
	/**
	 * Тип всплывающего окна
	 * @var string
	 */
	var $_popupType;
	
	/**
	 * Подключать ли ява-скрипты
	 *
	 * @var boolean
	 */
	var $_linkScripts;
	
	/**
	* Constructor
	* @param object $subject The object to observe
	* @param object $params  The object that holds the plugin parameters
	*/
	function plgContentMavikThumbnails( &$subject, $params )
	{
		parent::__construct( $subject, $params );
		$this->_thumbPath =	$this->params->def('thumbputh', 'plugins/content/mavikthumbnails/thumbnails');
		$this->_popupType =	$this->params->def('popuptype', 'slimbox');
		$this->_linkScripts = $this->params->def('link_scripts', 1);
	}
	
	/**
	* Method is called by the view
	* @param 	object		The article object.  Note $article->text is also available
	* @param 	object		The article params
	* @param 	int		The 'page' number
	*/
	function onPrepareContent( &$article, &$params, $limitstart )
	{
		global $mainframe;
		switch ($this->_popupType)
		{
			case 'slimbox':
				// Подключить библиотеку slimbox
				$document = &JFactory::getDocument();
				$document->addScript('media/system/js/mootools.js');
				if ($this->_linkScripts) {
					$document->addScript('plugins/content/mavikthumbnails/slimbox/js/slimbox.js');
					$document->addStyleSheet('plugins/content/mavikthumbnails/slimbox/css/slimbox.css');
				}
				break;
			default:
				// Подключить стандартную библиотеку модальных окон
				JHTML::_('behavior.modal');
		}
		// Найти в тексте изображения и заменить на иконки
		$regex = '#<img\s.*?>#';
		$article->text = preg_replace_callback($regex, array($this, "imageReplacer"), $article->text);
		return true;
	}

/**
 * Преобразует img-тег в html-код иконки 
 * @param array $matches
 * @return string
 */
	function imageReplacer(&$matches)
	{
		// Разобрать img-тег
		$newImgStr = $imgStr = $matches[0];
		// Подправить, если необходимо до соответсвия xml
		if ( substr($imgStr,-2,2) != '/>' ) $imgStr = str_replace('>', '/>', $imgStr);
		$imgStr = preg_replace('/(\w+)\s*=\s*([^"\'].*?)\s/', "$1=\"$2\" ", $imgStr);
		$xml = new JSimpleXML(); 
		$xml->loadString($imgStr);
		$this->_img =& $xml->document;
		//var_dump($this->_img);
		if(is_object($this->_img)) $this->_imgAtributes = $this->_img->attributes();
		 
		// Проверить необходимость замены - нужна ли иконка?
		// Прежде чем обращатья к функциям GD, проверяются атрибуты тега.
		if ( !empty($this->_imgAtributes['width']) || !empty($this->_imgAtributes['height']) )
		{
			$this->_origImgName = $this->_imgAtributes['src'];
			$this->_origImgName = $this->urlToFile($this->_origImgName);
				
			$this->_origImgSize = @getimagesize($this->_origImgName);
			$origImgW = $this->_origImgSize[0];
			$origImgH = $this->_origImgSize[1];
			
			if (( @$this->_imgAtributes['width'] && @$this->_imgAtributes['width'] < $origImgW ) ||	( @$this->_imgAtributes['height'] && @$this->_imgAtributes['width'] < $origImgW ))
			{
				// Заменить изображение на иконку
				$newImgStr = $this->createThumb();
			}
		}
		return $newImgStr;
	}
	
	/**
	 * Создает иконку, если она еще не существует.
	 * @return string html-код иконки
	 */
	function createThumb()
	{
		// Доопределить размеры, если необходимо
		if (empty($this->_imgAtributes['width'])) $this->_imgAtributes['width'] = intval($this->_imgAtributes['height'] * $this->_origImgSize[0] / $this->_origImgSize[1]); 
		if (empty($this->_imgAtributes['height'])) $this->_imgAtributes['height'] = intval($this->_imgAtributes['width'] * $this->_origImgSize[1] / $this->_origImgSize[0]);
		// Сформировать путь к иконке
		$thumbName = str_replace(array('/','\\',':'), '-', $this->_origImgName); 
		$thumbName = $this->_imgAtributes['width'] . 'x' . $this->_imgAtributes['height'] . '-' . $thumbName;
		$thumbPath = JPATH_BASE . DS . $this->_thumbPath . DS . $thumbName; 
		// Если иконки не существует - создать
		if (!file_exists($thumbPath))
		{
			// Определить расширение оригинального изображения
			$pathinfo = pathinfo($this->_origImgName);
			$ext = JString::strtolower($pathinfo['extension']); 
			// В зависимости от этого создать объект изобразения
			switch ($ext)
			{
				case 'jpg':
				case 'jpeg':
					$orig = imagecreatefromjpeg($this->_origImgName);
					break;
				case 'png':
					$orig = imagecreatefrompng($this->_origImgName);
					break;
				case 'gif':
					$orig = imagecreatefromgif($this->_origImgName);
					break;
				default:
					// Если расширение не поддерживается - вернуть тег без изменений
					return $this->_img->toString();
			}
			// Создать объект иконки
			$thumb = imagecreatetruecolor($this->_imgAtributes['width'], $this->_imgAtributes['height']);
		    // Обработать прозрачность
    		$transparent_index = imagecolortransparent($orig);
    		if ($transparent_index >= 0)
    		{
				// без альфа-канала
    			$t_c = imagecolorsforindex($orig, $transparent_index);
        		$transparent_index = imagecolorallocate($orig, $t_c['red'], $t_c['green'], $t_c['blue']);
        		imagecolortransparent($thumb, $transparent_index);
    		} else {
				// с альфа-каналом
				imagealphablending ( $thumb, false );
				imagesavealpha ( $thumb, true );
				$transparent = imagecolorallocatealpha ( $thumb, 255, 255, 255, 127 );
				imagefilledrectangle ( $thumb, 0, 0, $this->_imgAtributes['width'], $this->_imgAtributes['height'], $transparent );
    		}
			// Создать превью
    		imagecopyresampled($thumb, $orig, 0, 0, 0, 0, $this->_imgAtributes['width'], $this->_imgAtributes['height'], $this->_origImgSize[0],$this->_origImgSize[1]);
			// Записать иконку в файл
			switch ($ext)
			{
				case 'jpg':
				case 'jpeg':
					imagejpeg($thumb, $thumbPath, 80);
					break;
				case 'png':
					imagepng($thumb, $thumbPath);
					break;
				case 'gif':
					imagegif($thumb, $thumbPath);
			}
			imagedestroy($orig);
			imagedestroy($thumb);
		}
		$this->_img->addAttribute('src', $this->_thumbPath . '/' . $thumbName);
		$title = @$this->_imgAtributes['title'];
		if (empty($title) && !empty($this->_imgAtributes['alt'])) $title = $this->_imgAtributes['alt'];
		$title = htmlspecialchars($title); 
		switch ($this->_popupType)
		{
			case 'slimbox':
				$result = '<a class="thumbnail" href="' . $this->_imgAtributes['src'] . '" rel="lightbox" title="' . $title . '" target="_blank">' . $this->_img->toString() . '</a>';
				break;
			default:
				$result = '<a class="modal thumbnail" href="'. $this->_imgAtributes['src'] .'" rel="{handler: \'image\', marginImage: {x: 50, y: 50}}">' . $this->_img->toString() . '</a>'; 		
		}
		return $result;
	}
	
	/**
	 * Преобразует url-путь в путь к файлу
	 * если хост в url совпадает с url сайта,
	 * иначе оставляет без изменений
	 *
	 * @param string $url
	 */
	function urlToFile($url)
	{
		$siteUri = JFactory::getURI();
		$imgUri = JURI::getInstance($url);
		
		$siteHost = $siteUri->getHost();
		$imgHost = $imgUri->getHost();
		// игнорировать www при сверке хостов 
		$siteHost = preg_replace('/^www\./', '', $siteHost);
		$imgHost = preg_replace('/^www\./', '', $imgHost);
		if (empty($imgHost) || $imgHost == $siteHost) {
			$imgPath = $imgUri->getPath(); 
			// если путь к изображению абсолютный от корня домена (начинается со слеша),
			// преобразовать его в относительный от базового адреса сайта
			if ($imgPath[0] == '/')	{
				$siteBase = $siteUri->base();
				$dirSite = substr($siteBase, strpos($siteBase, $siteHost) + strlen($siteHost));
				$url = substr($imgPath, strlen($dirSite));
			}
			$url = str_replace('/', DS, $url);
		}
		return $url;
	}
}
